@@ -334,6 +334,10 @@ class XsdSimpleType(XsdType, ValidationMixin):
         else:
             return self.base_type.is_derived(other, derivation)
 
+    def is_dynamic_consistent(self, other):
+        return other is self.any_type or other is self.any_simple_type or self.is_derived(other) or \
+            hasattr(other, 'member_types') and any(self.is_derived(mt) for mt in other.member_types)
+
     def normalize(self, text):
         """
         Normalize and restrict value-space with pre-lexical and lexical facets.
@@ -513,28 +517,25 @@ class XsdAtomicBuiltin(XsdAtomic):
             yield self.decode_error(validation, obj, self.to_python,
                                     reason="value is not an instance of {!r}".format(self.instance_types))
 
-        if self.name == XSD_ID:
-            try:
-                id_map = kwargs['id_map']
-            except KeyError:
-                pass
-            else:
-                try:
-                    id_map[obj] += 1
-                except TypeError:
-                    id_map[obj] = 1
-
-                if id_map[obj] > 1 and '_skip_id' not in kwargs:
-                    yield self.validation_error(validation, "Duplicated xsd:ID value {!r}".format(obj))
-
-        elif self.name == XSD_IDREF:
+        if self.name == XSD_IDREF:
             try:
                 id_map = kwargs['id_map']
             except KeyError:
                 pass
             else:
                 if obj not in id_map:
-                    id_map[obj] = kwargs.get('node', 0)
+                    id_map[obj] = 0
+
+        elif self.name == XSD_ID and kwargs.get('level') != 0:
+            try:
+                id_map = kwargs['id_map']
+            except KeyError:
+                pass
+            else:
+                if not id_map[obj]:
+                    id_map[obj] = 1
+                else:
+                    yield self.validation_error(validation, "Duplicated xsd:ID value {!r}".format(obj))
 
         if validation == 'skip':
             try:
@@ -870,7 +871,8 @@ class XsdUnion(XsdSimpleType):
         return all(mt.is_list() for mt in self.member_types)
 
     def is_dynamic_consistent(self, other):
-        return other.is_derived(self) or hasattr(other, 'member_types') and \
+        return other is self.any_type or other is self.any_simple_type or \
+            other.is_derived(self) or hasattr(other, 'member_types') and \
             any(mt1.is_derived(mt2) for mt1 in other.member_types for mt2 in self.member_types)
 
     def iter_components(self, xsd_classes=None):
